//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.shipping.item;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.torque.TorqueException;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.Order;
import com.konakart.app.ShippingQuote;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.shipping.BaseShippingModule;
import com.konakart.bl.modules.shipping.ShippingInfo;
import com.konakart.bl.modules.shipping.ShippingInterface;
import com.workingdogs.village.DataSetException;

/**
 * This shipping module implements a flat rate per item which is set in the configuration property
 * MODULE_SHIPPING_ITEM_COST . If MODULE_SHIPPING_ITEM_TAX_CLASS is greater than zero, then tax is
 * added if the shipping address is in a taxable zone. The handling charge defined by
 * MODULE_SHIPPING_ITEM_HANDLING is also added.
 * 
 */
public class Item extends BaseShippingModule implements ShippingInterface
{
    private static int sortOrder = -1;

    private static int taxClass;

    private static int zone;

    private static BigDecimal cost;

    private static BigDecimal handling;

    private static String code = "item";

    private static String icon = "";

    private static String bundleName = BaseModule.basePackage + ".shipping.item.Item";

    private static HashMap<Locale,ResourceBundle> resourceBundleMap = new HashMap<Locale,ResourceBundle>();

    private static String mutex = "itemMutex";

    // Configuration Keys

    private final static String MODULE_SHIPPING_ITEM_STATUS = "MODULE_SHIPPING_ITEM_STATUS";

    private final static String MODULE_SHIPPING_ITEM_COST = "MODULE_SHIPPING_ITEM_COST";

    private final static String MODULE_SHIPPING_ITEM_HANDLING = "MODULE_SHIPPING_ITEM_HANDLING";

    private final static String MODULE_SHIPPING_ITEM_TAX_CLASS = "MODULE_SHIPPING_ITEM_TAX_CLASS";

    private final static String MODULE_SHIPPING_ITEM_ZONE = "MODULE_SHIPPING_ITEM_ZONE";

    private final static String MODULE_SHIPPING_ITEM_SORT_ORDER = "MODULE_SHIPPING_ITEM_SORT_ORDER";

    // Message Catalogue Keys

    private final static String MODULE_SHIPPING_ITEM_TEXT_TITLE = "module.shipping.item.text.title";

    private final static String MODULE_SHIPPING_ITEM_TEXT_DESCRIPTION = "module.shipping.item.text.description";

    private final static String MODULE_SHIPPING_ITEM_TEXT_WAY = "module.shipping.item.text.way";

    /**
     * Constructor
     * @param eng 
     * 
     * @throws DataSetException
     * @throws KKException
     * @throws TorqueException
     * 
     */
    public Item(KKEngIf eng) throws TorqueException, KKException, DataSetException
    {

        super.init(eng);

        // Create the static maps from the configuration info
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * From the ShippingCountry we find the iso code of the country and determine its zone.
     * 
     * @param order
     * @return Returns a ShippingQuote object
     * @throws KKException
     */
    public ShippingQuote getQuote(Order order, ShippingInfo info) throws Exception
    {
        // Throws an exception if there are no physical products. They may be all digital download
        // products.
        checkForProducts(info);

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        // Get a partially filled ShippingQuote object
        ShippingQuote quote = this.getShippingQuote(rb);

        /*
         * The global parameter zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Create the return string
        StringBuffer retTextBuf = new StringBuffer();
        retTextBuf.append(rb.getString(MODULE_SHIPPING_ITEM_TEXT_WAY));
        quote.setResponseText(retTextBuf.toString());

        // Return the cost
        BigDecimal orderCost = cost.multiply(new BigDecimal(info.getNumProducts()));
        quote.setCost(orderCost);
        quote.setHandlingCost(handling);
        BigDecimal costPlusHandling = orderCost.add(handling);
        if (taxClass > 0)
        {
            quote.setTax(getEng().getTax(costPlusHandling, info.getDeliveryCountry().getId(), info
                    .getDeliveryZone().getZoneId(), taxClass));
            quote.setTotalExTax(costPlusHandling);
            quote.setTotalIncTax(quote.getTax().add(costPlusHandling));
        } else
        {
            quote.setTax(new BigDecimal(0));
            quote.setTotalExTax(costPlusHandling);
            quote.setTotalIncTax(costPlusHandling);
        }
        return quote;
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_SHIPPING_ITEM_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_SHIPPING_ITEM_TAX_CLASS);
        if (conf == null)
        {
            taxClass = 0;
        } else
        {
            taxClass = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_SHIPPING_ITEM_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_SHIPPING_ITEM_COST);
        if (conf == null)
        {
            cost = new BigDecimal(0);
        } else
        {
            cost = new BigDecimal(conf.getValue());
        }

        conf = getEng().getConfiguration(MODULE_SHIPPING_ITEM_HANDLING);
        if (conf == null)
        {
            handling = new BigDecimal(0);
        } else
        {
            handling = new BigDecimal(conf.getValue());
        }
    }

    /**
     * 
     * @param rb
     * @return A ShippingQuote object
     * @throws KKException
     */
    private ShippingQuote getShippingQuote(ResourceBundle rb) throws KKException
    {

        ShippingQuote quote = new ShippingQuote();

        // Populate some attributes from static data
        quote.setCode(code);
        quote.setSortOrder(sortOrder);
        quote.setIcon(icon);
        quote.setTaxClass(taxClass);

        // Populate locale specific attributes from the resource bundle
        quote.setDescription(rb.getString(MODULE_SHIPPING_ITEM_TEXT_DESCRIPTION));
        quote.setTitle(rb.getString(MODULE_SHIPPING_ITEM_TEXT_TITLE));

        return quote;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_SHIPPING_ITEM_STATUS);
    }

}
